from pyembroidery.EmbConstant import *

READ_FILE_IN_TEXT_MODE = True


def read(f, out, settings=None):
    import csv
    csv_reader = csv.reader(f, delimiter=',')
    command_dict = get_command_dictionary()
    for row in csv_reader:
        if len(row) == 0:
            continue
        if "*" in row[0]:
            command = command_dict[row[2]]
            if len(row) == 3:
                out.add_command(command)
            else:
                out.add_stitch_absolute(command, float(row[3]), float(row[4]))
        elif "#" in row[0]:
            continue
        elif "@" in row[0]:
            if len(row) != 3:
                continue
            out.metadata(row[1], row[2])
        elif "$" in row[0]:
            thread_add = {}
            if len(row) == 7 and len(row[2]) <= 3 and len(row[3]) <= 3 and len(row[4]) <= 3:
                # This is an embroidermodder csv file, I changed the colors and added more details.
                # [THREAD_NUMBER], [RED], [GREEN], [BLUE], [DESCRIPTION], [CATALOG_NUMBER]\"\n");
                thread_add["rgb"] = (int(row[2]), int(row[3]), int(row[4]))
                thread_add["description"] = row[5]
                thread_add["catalog"] = row[6]
            else:
                try:
                    thread_add["rgb"] = row[2]
                except IndexError:
                    pass
                try:
                    thread_add["name"] = row[3]
                except IndexError:
                    pass
                try:
                    thread_add["brand"] = row[4]
                except IndexError:
                    pass
                try:
                    thread_add["catalog"] = row[5]
                except IndexError:
                    pass
                try:
                    thread_add["details"] = row[6]
                except IndexError:
                    pass
                try:
                    thread_add["weight"] = row[7]
                except IndexError:
                    pass
            out.add_thread(thread_add)


def get_command_dictionary():
    return {
        "NO_COMMAND": NO_COMMAND,
        "STITCH": STITCH,
        "JUMP": JUMP,
        "TRIM": TRIM,
        "STOP": STOP,
        "END": END,
        "SLOW": SLOW,
        "FAST": FAST,
        "COLOR_CHANGE": COLOR_CHANGE,
        "SEQUIN_MODE": SEQUIN_MODE,
        "SEQUIN_EJECT": SEQUIN_EJECT,
        "SEW_TO": SEW_TO,
        "NEEDLE_AT": NEEDLE_AT,
        "STITCH_BREAK": STITCH_BREAK,
        "SEQUENCE_BREAK": SEQUENCE_BREAK,
        "COLOR_BREAK": COLOR_BREAK,
        "TIE_ON": TIE_ON,
        "TIE_OFF": TIE_OFF,
        "FRAME_EJECT": FRAME_EJECT,
        "MATRIX_TRANSLATE": MATRIX_TRANSLATE,
        "MATRIX_SCALE": MATRIX_SCALE,
        "MATRIX_ROTATE": MATRIX_ROTATE,
        "MATRIX_RESET": MATRIX_RESET,
        "OPTION_ENABLE_TIE_ON": OPTION_ENABLE_TIE_ON,
        "OPTION_ENABLE_TIE_OFF": OPTION_ENABLE_TIE_OFF,
        "OPTION_DISABLE_TIE_ON": OPTION_DISABLE_TIE_ON,
        "OPTION_DISABLE_TIE_OFF": OPTION_DISABLE_TIE_OFF,
        "OPTION_MAX_STITCH_LENGTH": OPTION_MAX_STITCH_LENGTH,
        "OPTION_MAX_JUMP_LENGTH": OPTION_MAX_JUMP_LENGTH,
        "OPTION_IMPLICIT_TRIM": OPTION_IMPLICIT_TRIM,
        "OPTION_EXPLICIT_TRIM": OPTION_EXPLICIT_TRIM,
        "CONTINGENCY_NONE": CONTINGENCY_NONE,
        "CONTINGENCY_JUMP_NEEDLE": CONTINGENCY_JUMP_NEEDLE,
        "CONTINGENCY_SEW_TO": CONTINGENCY_SEW_TO,
    }



from .ReadHelper import read_int_8, read_int_16le


def find_extends(stitches):
    min_x = float('inf')
    min_y = float('inf')
    max_x = -float('inf')
    max_y = -float('inf')

    for stitch in stitches:
        if stitch[0] > max_x:
            max_x = stitch[0]
        if stitch[0] < min_x:
            min_x = stitch[0]
        if stitch[1] > max_y:
            max_y = stitch[1]
        if stitch[1] < min_y:
            min_y = stitch[1]
    return min_x, min_y, max_x, max_y


def read_pmv_stitches(f, out, settings=None):
    """PMV files are stitch files, not embroidery."""
    px = 0
    # stitches = []
    while True:
        stitch_count = read_int_16le(f)
        block_length = read_int_16le(f)
        if block_length is None:
            return
        if block_length >= 256:
            break
        if stitch_count == 0:
            continue
        for i in range(0, stitch_count):
            x = read_int_8(f)
            y = read_int_8(f)
            if y > 16:
                y = -(32 - y)  # This is 5 bit signed number.
            if x > 32:
                x = -(64 - x)  # This is a 6 bit signed number.
            x *= 2.5
            y *= 2.5
            dx = x
            out.stitch_abs(px + x, y)  # This is a hybrid relative, absolute value.
            px += dx
            # stitches.append((x, y))
    out.end()
    # f.seek(0x10, 1)  # 16 bytes
    # block_end = read_int_16le(f)
    # if block_end != 256:
    #     return
    # steps = []
    # dunno0 = read_int_8(f)
    # dunno1 = read_int_8(f)
    # length_steps = read_int_8(f)
    # steps_size = read_int_8(f)
    # for i in range(0, steps_size):
    #     x = read_int_16le(f)
    #     y = read_int_16le(f)
    #     if x is None or y is None:
    #         break
    #     steps.append((x, y))
    # width_units = read_int_8(f)
    # steps2_size = read_int_8(f)
    # steps2 = []
    # for i in range(0, steps2_size):
    #     x = read_int_16le(f)
    #     y = read_int_16le(f)
    #     if x is None or y is None:
    #         break
    #     steps2.append((x, y))
    # dunno4 = read_int_16le(f)  # seems to be 0x12.
    # f.seek(0x10, 1)  # 16 bytes
    # # EOF - This should be End of File.
    # none_bytes = read_int_8(f)
    # if none_bytes is None:
    #     pass
    #
    # extends = find_extends(stitches)
    # print(f)
    # print("Stitches: Total ", len(stitches), " : ", stitches)
    # print("Unknown0:", dunno0)
    # print("Unknown1:", dunno1)
    # print("Length Position:", length_steps)
    # print("Length Lookup: ", len(steps), " : ", steps)
    # print("Length value:", steps[length_steps])
    # length_max = extends[2] - extends[0]
    # width_max = extends[3] - extends[1]
    # print("Max Length:", length_max)
    # print("Max dx+:", extends[2])
    # print("Max dx-:", extends[0])
    # print("Width Position:", width_units)
    # print("Width Lookup:", len(steps2), " : ", steps2)
    # print("Width value:", steps2[width_units])
    # print("Max Width:", width_max)
    #
    # print("Unknown4:", dunno4)

    out.end()


def read(f, out, settings=None):
    f.seek(0x64, 0)
    read_pmv_stitches(f, out)



def read_barudan_dat(f, out):
    stitched_yet = False
    count = 0
    while True:
        count += 1
        byte = bytearray(f.read(3))
        if len(byte) != 3:
            break

        ctrl = byte[0]
        y = -byte[1]
        x = byte[2]

        if ctrl & 0x80 == 0:
            # This bit should always be set, must be other dat type.
            return False
        if ctrl & 0x40 != 0:
            y = -y
        if ctrl & 0x20 != 0:
            x = -x
        if (ctrl & 0b11111) == 0:
            stitched_yet = True
            out.stitch(x, y)
            continue
        if (ctrl & 0b11111) == 1:
            out.move(x, y)
            continue
        if ctrl == 0xF8:
            break
        if ctrl == 0xE7:
            out.trim()
            continue
        if ctrl == 0xE8:
            if count > 1:
                out.stop()
            continue
        if 0xE9 <= ctrl < 0xF8:
            needle = ctrl - 0xE8
            if stitched_yet:
                out.color_change()
            continue
        break  # Uncaught Control
    out.end()
    return True


def read_sunstar_dat_stitches(f, out):
    count = 0
    while True:
        count += 1
        byte = bytearray(f.read(3))
        if len(byte) != 3:
            break
        x = byte[0] & 0x7F
        y = byte[1] & 0x7F
        if byte[0] & 0x80:
            x = -x
        if byte[1] & 0x80:
            y = -y
        y = -y
        ctrl = byte[2]
        if ctrl == 0x07:
            out.stitch(x, y)
            continue
        if ctrl == 0x04:
            out.move(x, y)
            continue
        if ctrl == 0x80:
            out.trim()
            if x != 0 or y != 0:
                out.move(x, y)
            continue
        if ctrl == 0x87:
            out.color_change()
            if x != 0 or y != 0:
                out.move(x, y)
            continue
        if ctrl == 0x84:  # Initialized info.
            out.stitch(x, y)
            continue
        elif ctrl == 0:
            break
        break  # Uncaught Control
    out.end()


def read_sunstar_dat(f, out):
    # f.seek(0x02, 0)
    # stitches = read_int_16le(f)
    f.seek(0x100, 0)
    read_sunstar_dat_stitches(f, out)


def read(f, out, settings=None):
    if not read_barudan_dat(f, out):
        f.seek(0, 0)
        read_sunstar_dat(f, out)
